# 枚举类型

> 原文： [https://docs.oracle.com/javase/tutorial/java/javaOO/enum.html](https://docs.oracle.com/javase/tutorial/java/javaOO/enum.html)

_ 枚举类型 _ 是一种特殊数据类型，它使变量成为一组预定义常量。变量必须等于为其预定义的值之一。常见示例包括罗盘方向（NORTH，SOUTH，EAST 和 WEST 的值）和星期几。

因为它们是常量，所以枚举类型字段的名称是大写字母。

在 Java 编程语言中，您可以使用`enum`关键字定义枚举类型。例如，您可以将星期几的枚举类型指定为：

```
public enum Day {
    SUNDAY, MONDAY, TUESDAY, WEDNESDAY,
    THURSDAY, FRIDAY, SATURDAY 
}

```

您需要在需要表示一组固定常量时使用枚举类型。这包括自然枚举类型，例如太阳系中的行星和数据集，您可以在编译时知道所有可能的值 - 例如，菜单上的选项，命令行标志等。

以下是一些代码，向您展示如何使用上面定义的`Day`枚举：

```
public class EnumTest {
    Day day;

    public EnumTest(Day day) {
        this.day = day;
    }

    public void tellItLikeItIs() {
        switch (day) {
            case MONDAY:
                System.out.println("Mondays are bad.");
                break;

            case FRIDAY:
                System.out.println("Fridays are better.");
                break;

            case SATURDAY: case SUNDAY:
                System.out.println("Weekends are best.");
                break;

            default:
                System.out.println("Midweek days are so-so.");
                break;
        }
    }

    public static void main(String[] args) {
        EnumTest firstDay = new EnumTest(Day.MONDAY);
        firstDay.tellItLikeItIs();
        EnumTest thirdDay = new EnumTest(Day.WEDNESDAY);
        thirdDay.tellItLikeItIs();
        EnumTest fifthDay = new EnumTest(Day.FRIDAY);
        fifthDay.tellItLikeItIs();
        EnumTest sixthDay = new EnumTest(Day.SATURDAY);
        sixthDay.tellItLikeItIs();
        EnumTest seventhDay = new EnumTest(Day.SUNDAY);
        seventhDay.tellItLikeItIs();
    }
}

```

输出是：

```
Mondays are bad.
Midweek days are so-so.
Fridays are better.
Weekends are best.
Weekends are best.

```

Java 编程语言枚举类型比其他语言中的对应类型更强大。 `enum`声明定义 _ 类 _（称为 _ 枚举类型 _）。枚举类主体可以包括方法和其他字段。编译器在创建枚举时会自动添加一些特殊方法。例如，它们有一个静态`values`方法，该方法返回一个数组，该数组按照声明的顺序包含枚举的所有值。此方法通常与 for-each 构造结合使用，以迭代枚举类型的值。例如，下面`Planet`类示例中的代码迭代太阳系中的所有行星。

```
for (Planet p : Planet.values()) {
    System.out.printf("Your weight on %s is %f%n",
                      p, p.surfaceWeight(mass));
}

```

* * *

**Note:** _All_ enums implicitly extend `java.lang.Enum`. Because a class can only extend one parent (see [Declaring Classes](classdecl.html)), the Java language does not support multiple inheritance of state (see [Multiple Inheritance of State, Implementation, and Type](../IandI/multipleinheritance.html)), and therefore an enum cannot extend anything else.

* * *

在以下示例中，`Planet`是枚举类型，表示太阳系中的行星。它们具有恒定的质量和半径属性。

每个枚举常量都声明为质量和半径参数的值。创建常量时，这些值将传递给构造函数。 Java 要求在任何字段或方法之前首先定义常量。此外，当存在字段和方法时，枚举常量列表必须以分号结尾。

* * *

**Note:** The constructor for an enum type must be package-private or private access. It automatically creates the constants that are defined at the beginning of the enum body. You cannot invoke an enum constructor yourself.

* * *

除了它的属性和构造函数之外，`Planet`还有一些方法可以让你检索每个行星上一个物体的表面重力和重量。这是一个示例程序，它可以减轻地球上的重量（在任何单位）并计算并打印所有行星上的重量（在同一单位）：

```
public enum Planet {
    MERCURY (3.303e+23, 2.4397e6),
    VENUS   (4.869e+24, 6.0518e6),
    EARTH   (5.976e+24, 6.37814e6),
    MARS    (6.421e+23, 3.3972e6),
    JUPITER (1.9e+27,   7.1492e7),
    SATURN  (5.688e+26, 6.0268e7),
    URANUS  (8.686e+25, 2.5559e7),
    NEPTUNE (1.024e+26, 2.4746e7);

    private final double mass;   // in kilograms
    private final double radius; // in meters
    Planet(double mass, double radius) {
        this.mass = mass;
        this.radius = radius;
    }
    private double mass() { return mass; }
    private double radius() { return radius; }

    // universal gravitational constant  (m3 kg-1 s-2)
    public static final double G = 6.67300E-11;

    double surfaceGravity() {
        return G * mass / (radius * radius);
    }
    double surfaceWeight(double otherMass) {
        return otherMass * surfaceGravity();
    }
    public static void main(String[] args) {
        if (args.length != 1) {
            System.err.println("Usage: java Planet <earth_weight>");
            System.exit(-1);
        }
        double earthWeight = Double.parseDouble(args[0]);
        double mass = earthWeight/EARTH.surfaceGravity();
        for (Planet p : Planet.values())
           System.out.printf("Your weight on %s is %f%n",
                             p, p.surfaceWeight(mass));
    }
}

```

如果从参数为 175 的命令行运行`Planet.class`，则会得到以下输出：

```
$ java Planet 175
Your weight on MERCURY is 66.107583
Your weight on VENUS is 158.374842
Your weight on EARTH is 175.000000
Your weight on MARS is 66.279007
Your weight on JUPITER is 442.847567
Your weight on SATURN is 186.552719
Your weight on URANUS is 158.397260
Your weight on NEPTUNE is 199.207413

```